<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/datalist/RangeDataList.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Authors: Michael Clay <claym@osuosl.org>                               |
// |          Frederic Wenzel <fred@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: RangeDataList.php 4838 2006-09-27 17:16:07Z morgabra $

/**
  * RangeDataList.php - contains the {@link RangeDataList} class
  *
  * @author $Author: morgabra $
  * @filesource
  * @package    htdocs
  * @subpackage datalists
  */

require_once(DAOEXT.'/range.php');
require_once(HTCLASS.'/DB_DataObjectDataList.php');
require_once(HTCLASS.'/DB_DataObjectDataListSource.php');

/**
  * Table listing for IP Ranges
  *
  * @link   http://phphtmllib.newsblob.com/javadoc.php phpHtmlLib API Documentation
  * @link   http://phphtmllib.newsblob.com/doc/phpHtmlLib/DefaultGUIDatalist.html
            DefaultGUIDataList API Documentation
  * @package    htdocs
  * @subpackage datalists
  */
class RangeDataList extends DB_DataObjectDataList
{

//{{{ vars
    // convert the search value to int when searching for ips?
    var $search_convert_ip2int = false;
//}}}

//{{{   get_data_source()

    function get_data_source()
    {
        global $user;
    
        $rng = new Range();
        $a = strtolower($_REQUEST['action']);
        if(($a != "show_admin") || !$user->is_admin()) {

            $zr = DB_DataObject::Factory('zone_range');
            $zr->zone = $_SESSION['zone'];
            $rng->joinAdd($zr);

        }
        
        $src = &new DB_DataObjectDataListSource($rng);
        $this->set_data_source($src);
        $this->_default_orderby = 'start';
    }

//}}}
//{{{   _user_setup()

    function _user_setup()
    {
        global $user;

        $this->add_header_item(_("Name"), '20%','name',SORTABLE,SEARCHABLE);
        $this->add_header_item(_("Range"), '20%', 'start', SORTABLE, SEARCHABLE);
        $this->add_header_item(_("Free Addresses"), '10%');
        $this->add_header_item(_("Type"), '10%', 'type', SORTABLE, null);
        if($user->is_admin()) $this->add_header_item(_("Zone"),'20%');
        $this->add_header_item(_("Action"),'20%');
        if(!$user->is_admin()) $this->_datasource->setup_db_options(array('zone'=>$_SESSION['zone']));
    }

//}}}
//{{{   build_column_item()
    
    function build_column_item($row_data, $col_name)
    {
        global $user;
        switch ($col_name) {
        
        case _('Name'):
            $obj = html_a(web_root()."range.php?action=show&amp;id=".$row_data['id'],$row_data['name']);
            break;
        
        case _('Range'):
            $obj = container($row_data['start']," to ", $row_data['end']);
            break;
        case _('Free Addresses');
            $dbdo = $this->_datasource;
            $dbdo = $dbdo->get_dbdo();
            $range = new Range($dbdo);
            $count = $dbdo->display_free_adresses();
            $obj = ($count ? $count : "0");
            break;
        case _('Action'):
            $obj = container();
            $obj = Module::apply_filter('action_module', $obj, $row_data, 'range');
            break;
        case _("Zone"):
            $znr = DB_DataObject::Factory('zone_range');
            $znr->range = $row_data['id'];
            $res = $znr->find();
            switch($res) {
            case 1:
                $znr->fetch();
                $zn = new Zone($znr->zone);
                $obj = $zn->name;
                break;
            case 0:
                $obj = _("No Zones");
                break;
            default:
                $array = array();
                while($znr->fetch()) {
                    $zn = new Zone($znr->zone);
                    $array[$zn->name] = '';
                }
                $obj = form_select('zones',$array,$_SESSION['zone']);
                break; 
            }
            break;
        default:
            $obj = parent::build_column_item($row_data, $col_name);
        
        }

        return $obj;
    }

//}}}
//{{{ data_prefetch()
function data_prefetch()
{
    if ($this->search_field() == 'start') {
        $this->search_convert_ip2int = true;
        parent::data_prefetch();
        $this->search_convert_ip2int = false;
    } else {
        parent::data_prefetch();
    }
}
//}}}
//{{{ search_value()
    function search_value()
    {
        // convert ip to int representation for ip search?
        if ($this->search_convert_ip2int && $this->search_field() == "start") { 
           return Net_IPv4::ip2double(parent::search_value());
        } else {
           return parent::search_value();
        }
    }
                                                                    
//}}}
}

?>
