<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/forms/HostDisplayForm.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// |            Zack Bartel  <bartel@osuosl.org>                            |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: HostDisplayForm.php 4606 2006-07-12 22:41:37Z morgabra $

/**
 *  HostDisplayForm.php - contains the {@link HostDisplayForm}, {@link MoveHostForm}
 *                        and {@link ZoneChangeForm} and {@link DeleteConfirm} class
 *
 *  @author $Author: morgabra $
 *  @filesource
 *  @package    htdocs
 *  @subpackage forms
 */

define("DYNAMIC", 0);
define("STATICIP",1);

require_once(FORMDIR."/HostCreateForm.php");

/**
 *  This is the form for editing hosts.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class HostDisplayForm extends HostCreateForm
{

/*Form Content Method Overloading*/
//{{{   form_init_data()

    /**
     *  This method sets the inital values for all the fun forms
     */
    function form_init_data()
    {
        global $id;

        $host = new Host($_REQUEST['id']);

        $array = $host->toArray();

        $init = $this->remap($array,FORM);
        $init['Domain'] = $host->domain; //one special case
        $init['Workgroup'] = $host->workgroup; //two special cases
        $init['Dynamic Range'] = $host->dynamic_range;//
        
        // enable or disable the dynamic range/ip fields
        // depending on which one applies to this host
        $el_dr = &$this->get_element(_("Dynamic Range"));
        $el_ip = &$this->get_element(_("IP Address"));
        if($host->dynamic_range == 0) {
            $this->set_element_value(_("Type"),"static");
            $el_dr->set_disabled(true);
            $el_ip->set_disabled(false);
        } else {
            $this->set_element_value(_("Type"), "dynamic");
            $el_dr->set_disabled(false);
            $el_ip->set_disabled(true);
        }
       
        foreach($init as $label => $val) {
            if($this->contains_element($label)) {
                $this->set_element_value($label,$val);
            }
        }

        $this->set_element_value(_("Comment"),"Host updated");
    }    

//}}}
//{{{   form_action()

    function form_action()
    {
        global $user;

        $res = false;
        switch($this->get_action())
        {
            case _("Submit"): //update button got pressed
                $host = new Host($_REQUEST['id']);
                $host->query("BEGIN"); 
                //$vals = $this->remap($this->get_all_element_values(), DBDO);
                $vals = $this->remap($_POST, DBDO);
                //Lets set the hosts values to the submitted ones.
                $host->setFrom($vals);
                $host->last_seen = time();
                $valid = $host->update(); //attempt to update
                
                if($valid === false) { //If validation failed due to error
                    $host->query("ROLLBACK");
                    $this->add_error(_("ERROR"),_("Failed to update host!"));
                }
                else { //otherwise life is good
                    $host->query("COMMIT");
                    //Make a record of the event
                    $host->write_history($_REQUEST[_('Comment')], ACTION_MODIFY, $user->username, $host->zone);
                    $res = true;
                }
             
                break;
            case _("Delete"):
                $res = true;
                break;
            default:
                $this->add_error(_("ERROR"),_("Unrecognized action"));
                $res = false;
                break;
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method is executed when the form_action method returns true.
     *  In this case it redirects the user to the host edit page with the new vals.
     */
    function form_success()
    {
        $div = html_div("success");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $hostlink = html_a(web_root()."host.php", _("[Return to Host List]"));
        $div->add(_("Host Successfully Updated"), html_br(), $indlink, $hostlink);
        return $div;
    }

//}}}
//{{{   form_hidden()
    /*
     *  Need to keep the action AND the id around
     *
     */
    function form_hidden()
    {
        $act = new FEHidden("action", $_REQUEST['action']);
        $id = new FEHidden("id", $_REQUEST['id']);
        $this->add_elements($act, $id); 
        return container($this->element_form("action"), $this->element_form("id"));

    }
//}}}
//{{{   form_backend_validation()

    /**
     *  This method is the one that performs the validation using a the DBDO's
     *  built in authentication methods.  It takes advantage of the remap function
     *  in order to maintain a relationship between the internal vars of the DBDO
     *  and the form values()
     *
     *  @return boolean $res    True if validation successful
     */
    function form_backend_validation()
    {
        $host = new Host($_REQUEST['id']); //make a host
        $array = $this->get_all_element_values(); // get the form values
        $host->setFrom($this->remap($array,DBDO)); //give them to the host
        $valid = $host->validate(); //validate the host with new info
        
        $res = true; //assume true

        //Comment block is a special case
        $comvalid = validate_comment($array[_("Comment")]);
        if($comvalid !== true) { //if the comment isn't valid
            $this->add_error(_("Comment"),$comvalid); //add error
            $res = false; //not valid form
        }

        //everything else
        if(is_array($valid)) {
            $errors = $this->remap($valid, FORM); //Remap the errors to forms
            foreach($errors as $name=>$state)
                if($state !== true) //true implies that part was valid
                    $this->add_error($name,$state); //add an error
            $res = false; //result is not valid form
        }

        return $res; //we're done
        
    }

//}}}

/*Helper Methods to generate arrays to populate the list boxes*/
//{{{   _domains()

    /**
     *  This method is a helper method that is here really jsut to clean up the
     *  code.  This method gets an array of domain ids indexed by their names.
     *  It is meant to populate the list boxes.
     *
     *  @return Array $domains  An array of domain ids indexed by name
     */
    function _domains()
    {
        global $user;
        
        $domains = array();
       
        //The fun part about this is zone_domain doesn't know the name...
        //Thus we do a magical join operation because the domain table does.
        $zd = DB_DataObject::Factory('zone_domain');
        if(!$user->is_admin())//If they're a full admin we don't care about zones
            $zd->zone = $_SESSION['zone'];
        $dom = DB_DataObject::Factory('domain');
        $dom->joinAdd($zd);
        $dom->orderBy('name');

        $dom->find();

        while($dom->fetch()) {
            $domains[$dom->name] = $dom->domain;
        }

        return $domains;
    }

//}}}
//{{{   _workgroups()

    /**
     *  This method gets the workgroups that are eligible targets for moving a
     *  host to.  The result is an array of workgroup names that are indexed by
     *  their database id.  This method is only to clean up code appearance.
     *
     *  @return Array   $workgroups  An array of workgroup ids indexed  by name.
     */
    function _workgroups()
    {
        global $user, $conf;

        $host = new Host($_REQUEST['id']);
        $workgroups = array();
        
        $zw = DB_DataObject::Factory('zone_workgroup');
        $zw->zone = $host->zone; 
        $wg = DB_DataObject::Factory('workgroup');
        $wg->joinAdd($zw);

        $wg->find();

        $workgroups[""] = ""; //add blank if they don't really want the thing.
        while($wg->fetch()) {
            $workgroups[$wg->name] = $wg->workgroup;
        }

        $wg = DB_DataObject::Factory('workgroup');
        $wg->name = $conf['general']['default_workgroup'];
        $wg->find(1);
        $workgroups[$wg->name] = $wg->id;

        return $workgroups;
    }

//}}}

}

/**
 *  This form allows a full admin to move a host across zones.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class MoveHostForm extends MaintainForm
{

//{{{   form_init_elements()

    /**
     *  This method intializes the fields for the user
     */
    function form_init_elements()
    {
        $dbdo = DB_DataObject::Factory('zone');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->selectAdd('name');
        $dbdo->orderBy('name ASC');
        $zoneary = array();
        
        $dbdo->find();
        while($dbdo->fetch()) {
            $zoneary[$dbdo->name] = $dbdo->id;
        }
        
        $zonelist = new FEListBox(_("Target Zone"));
        $zonelist->set_list_data($zoneary);
        
        $this->add_element($zonelist);
        $this->add_element(new FEHidden("action","edit"));
        $this->add_element(new FEHidden("id", $_REQUEST['id']));
        $this->set_required_marker(null);

        $this->set_action($SERVER["SCRIPT_NAME"]);
    }

//}}}
//{{{   form_init_data()

    /**
     *  This method sets the default value for the fields
     */
    function form_init_data()
    {
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->id = $_REQUEST['id'];
        $dbdo->selectAdd();
        $dbdo->selectAdd('zone');
        $dbdo->find(1);

        $this->set_element_value(_('Target Zone'),$dbdo->zone);
    }

//}}}
//{{{   form()

    /**
     *  This method is what lays the form out on the screen.
     *
     *  @return DIVTag  $div    A div that contains the form object
     */
    function form()
    {

        $div = html_div("targetzone");
        $table = html_table();
        
        /*This is the title to the sub table*/
        $tr = html_tr("even");
        $span = html_span("subtitle");
        $span->add(_("Move Host to Another Zone"));
        $tr->add($span);
        $table->add($span);
        
        /*Here goes the the actual form element*/
        $tr = html_tr("odd");
        $tr->add($this->element_label(_('Target Zone')),$this->element_form(_('Target Zone')));
        $table->add($tr);

        /*This would be the local submit button for all your submission needs*/
        $tr = html_tr("even");
        $tr->add("&nbsp;",$this->add_action(_("Move Host")));
        $table->add($tr);
        $div->add($table);
        $div->add($this->element_form("action"));
        $div->add($this->element_form("id"));
        
        return $div;
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what performs the action of performing the update
     *
     *  @return boolean     Update successful?
     */
    function form_action()
    {
        global $user;
        
        $host = new Host($_REQUEST['id']);
        $host->query("BEGIN");
        $oldzone = $host->zone;
        $host->setZone($this->get_element_value(_("Target Zone")));
        
        if($host->update() === false) { //false means something bad happened
            $host->query("ROLLBACK");
            $this->add_error("ERROR", "Update failed"); //That's not cool
            return false; //failure++
        } else {
            $host->query("COMMIT");
            $host->write_history(_("Moved Host from").' '.$oldzone.' '._("to").
                                ' '.$host->zone, ACTION_MODIFY, $user->username,
                                $host->zone);
        }
        return true; //we win!
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This is the validation method.
     *
     *  @return boolean     Is the zone valid?
     */
    function form_backend_validation()
    {
        $host = new Host($_REQUEST['id']);
        $host->setZone($_REQUEST['Target_Zone']); //our new target zone

        $errors = $host->validate(); //run validation to check for errors
        if($errors  !== true) { //if it wasn't true then it was the error array
            foreach($errors as $name=>$message) //iterate through
                if($message !== true)
                    $this->add_error($name, $message); //add the errors
            return false; //validation failed, see errors
        }
 
        return true; //validation successful
    }

//}}}

}

/**
 *  This form is used when a full admin has decided to move a host to a new zone.
 *  It offers the admin the choices of which new workgroup, domain and IP range to
 *  put it in.  This is because it cannot be assumed that the old settings apply
 *  to the new zone.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class ZoneChangeForm extends MaintainForm
{

/*Functions that overload form content*/
//{{{   form_init_elements()

    function form_init_elements()
    {
        $ipbox = new FEListBox(_("New Range"));
        $dombox = new FEListBox(_("New Domain"));
        $wgbox = new FEListBox(_("New Workgroup"));

        $ipbox->set_list_data($this->_freeips());
        $dombox->set_list_data($this->_domains());
        $wgbox->set_list_data($this->_workgroups());

        $this->add_element($ipbox);
        $this->add_element($dombox);
        $this->add_element($wgbox);

        $this->set_required_marker(null);
        $this->set_action($_SERVER['SCRIPT_NAME']);
    }
    
//}}}
//{{{   form()
    function form()
    {
        $host = new Host($_REQUEST['id']); //I need the host's name
        $zonedo = new Zone($_REQUEST['Target_Zone']); //target zone for name.
        $div = html_div("zonechangeform");
        $table = html_table();

        //Start our table by telling them the operation we're doing.
        $tr = html_tr("even");
        $tr->add(_('Moving Host ').$host->name._(' to Zone '). $zonedo->name);
        $table->add($tr);
        
        //used for alternating the row classes
        $class = array(0=>"even",
                        1=>"odd");
        $temp = 0;
        
        foreach($this->_elements as $label=>$element) {
            $tr = html_tr($class[++$temp%2]); //make a row and alternate the class
            $tr->add($this->element_label($label)); //add the label
            $tr->add($this->element_form($label)); //add the form
            $table->add($tr); //add the row to the table.
        }
        
        $tr = html_tr($class[++$temp%2]); //make one more row, need buttons =)
        $tr->add("&nbsp;",container($this->add_action(_("Move")),$this->add_action(_("Cancel"))));
        $table->add_row($tr); //add button row to table
        
        $div->add($table); //add table to div
        $this->add_element(new FEHidden("action","edit"));
        $this->add_element(new FEHidden("id",$_REQUEST['id']));
        $this->add_element(new FEHidden("Target_Zone", $_REQUEST[_('Target_Zone')]));
        $div->add($this->element_form("action"));
        $div->add($this->element_form("id"));
        $div->add($this->element_form("Target_Zone"));
        
        return $div; //we're out
    }

//}}}
//{{{   form_action()

    function form_action()
    {
        global $user;

        $valid = true;
        
        $ip = null; //this will be used as a flag;
        $ipval = null;

        //this is where we see if the range we got was static or dynamic
        $test = new Range($this->get_element_value(_("New Range")));
       
        if(!strcasecmp($test->type,"static")) {
            $ip     = STATICIP; 
            $ipval  = $test->get_free_addresses(1);
            $ipval  = $ipval[0];
        }
        else {
            $ip     =  DYNAMIC;
            $ipval  =  $test->id;
        }

        switch($_REQUEST['_form_action']) {
            case _("Cancel"):
                break;
            case _("Move"):
                
                $host = new Host($_REQUEST['id']);
                $host->query("BEGIN");
                $host->setDomain($this->get_element_value(_("New Domain")));
                if($ip === STATICIP)
                    $host->setIp($ipval);            
                else
                    $host->setDynamic_range($ipval);
                $host->setWorkgroup($this->get_element_value(_("New Workgroup")));
                $host->setZone($_REQUEST[_('Target_Zone')]);
                $valid = $host->update();
                if($valid === false){
                    $host->query("ROLLBACK");
                    $this->add_error(_("ERROR"), _("Host move failed!"));
                }
                else {
                    $host->query("COMMIT");
                    $host->write_history("Host moved to zone id ". 
                                      $_REQUEST[_('Target_Zone')], ACTION_MODIFY,
                                      $user->username, $host->zone);
                }
                break;
            default:
                $this->add_error(_("ERROR"),_("An invalid button was pressed!"));
                $valid = false;
        }

        return $valid;
    }

//}}}
//{{{   form_success()

    /**
     *  This method is executed if the form_action method returns true.
     *  In this case it just forwards them back to the host with its new vals.
     */
    function form_success()
    {
        header("Location: host.php?action=edit&amp;id=".$_REQUEST['id']);
    }

//}}}

/*Helper Functions to Populate the List Boxes*/
//{{{   _domains()
    
    /**
     *  This method is a helper method that is here really jsut to clean up the
     *  code.  This method gets an array of domain ids indexed by their names.
     *  It is meant to populate the list boxes.
     *
     *  @return Array $domains  An array of domain ids indexed by name
     */
    function _domains()
    {
        global  $user;
        
        $domains = array();
       
        //The fun part about this is zone_domain doesn't know the name...
        //Thus we do a magical join operation because the domain table does.
        $zd = DB_DataObject::Factory('zone_domain');
        $zd->zone = $_REQUEST['Target_Zone'];
        $dom = DB_DataObject::Factory('domain');
        $dom->orderBy("domain.name ASC");
        $dom->joinAdd($zd);
        $dom->find();

        while($dom->fetch()) {
            $domains[$dom->name] = $dom->domain;
        }

        return $domains;
    }

//}}}       
//{{{   _freeips()

    function _freeips()
    {
        $ranges = array();
        
        $zrng = DB_DataObject::Factory('zone_range');
        $zrng->zone = $_REQUEST[_('Target_Zone')];
        $rng = DB_DataObject::Factory('ranges');
        $rng->joinAdd($zrng);
        $rng->orderBy('ranges.start ASC');
        $rng->find();

        while($rng->fetch()) {
            $range = new Range($rng->range);
            if($range->get_free_addresses())
                $ranges[IP::int2ip($rng->start).' to ' .IP::int2ip($rng->end)] = $rng->range;
        }

        return $ranges;
    }
    
//}}}
//{{{   _workgroups()

    /**
     *  This method gets the workgroups that are eligible targets for moving a
     *  host to.  The result is an array of workgroup names that are indexed by
     *  their database id.  This method is only to clean up code appearance.
     *
     *  @return Array   $workgroups  An array of workgroup ids indexed  by name.
     */
    function _workgroups()
    {
        global  $conf;

        $workgroups = array();
        
        $zw = DB_DataObject::Factory('zone_workgroup');
        $zw->zone = $_REQUEST['Target_Zone']; 
        $wg = DB_DataObject::Factory('workgroup');
        $wg->joinAdd($zw);
        $wg->orderBy("workgroup.name ASC");
        $wg->find();

        while($wg->fetch()) {
            $workgroups[$wg->name] = $wg->workgroup;
        }

        $wg = DB_DataObject::Factory('workgroup');
        $wg->name = $conf['general']['default_workgroup'];
        $wg->find(1);
        $workgroups[$wg->name] = $wg->id;

        return $workgroups;
    }

//}}}

}

/**
 *  This class is the delete confirm page
 *
 *  @package    htdocs
 *  @subpackege forms
 */
class HostDeleteConfirm extends FormContent
{

//{{{   form()
    /**
     *  This method lays the form out for display.
     *
     *  @return DIVTag  $div    Div that contains the form
     */
    function form()
    {
        $host = new Host($_REQUEST['id']);
        $div = html_div("delconfirm");
        $cont = container();
        $cont->add(_("You are going to delete ").$host->name.".".$host->getDomain());
        $cont->add(html_br());
        $cont->add(_("Are you sure?"));
        $cont->add(html_br(),$this->add_action(_("Confirm")),$this->add_action(_("Cancel")));
        $cont->add($this->element_form("action"));
        $cont->add($this->element_form("id"));
        $div->add($cont);

        return $div;
    }

//}}}
//{{{   form_action

    /**
     *  This method defines the behavior for the form based on the button pressed
     *
     *  @return boolean     true if action was successful
     */
    function form_action()
    {
        global $user;
        
        switch(strtolower($this->get_action()))
        {
            case 'confirm':
                $host = new Host($_REQUEST['id']);
                $host->query("BEGIN");
    
                if($host->delete() === false) {
                    $host->query("ROLLBACK");
                    $this->add_error(_("ERROR"), _("Could not delete host"));
                }
                else    {
                    $host->query("COMMIT");
                    $host->write_history("Host ".$host->name.".".$host->getDomain(                                     ). " deleted",ACTION_DELETE, $user->username,
                                     $host->zone);

                }
   
                break;
            case 'cancel':
                header("Location: host.php?action=show");
                break;
            default:
                die_hard(_("Invalid form action"));
                break;
        }
        return true;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a message informing the user that the delete was
     *  successful.
     *
     *  @return DIVTag  $div    The div that contains the message
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a("index.php", "[Return to Home]");
        $hoslink = html_a("host.php", "[Return to Host List]");
        $div->add(container("Host deleted successfully!",html_br()));
        $div->add(container($indlink, $hoslink));
        return $div;
    }

//}}}
//{{{   form_init_elements()

    /**
     *  This method inits the elements that are able to be included in the form.
     */
    function form_init_elements()
    {
        $this->add_element(new FEHidden("action","delconfirm"));
        $this->add_element(new FEHidden("id", $_REQEST['id']));
        $this->set_action($_SERVER["SCRIPT_NAME"]);
    }

//}}}

}

?>
