<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel  <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: WorkgroupCreateForm.php 4606 2006-07-12 22:41:37Z morgabra $

/**
 *  WorkgroupCreateForm.php - contains the {@link WorkgroupCreateForm} class
 *
 *  @author     $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class is the workgroup creation form
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class WorkgroupCreateForm extends MaintainForm
{

//{{{   vars

    var $map = array("Name"=>"workgroup",
                    "Initial_Zone"=>"zone"
                    );

//}}}

//{{{   form_init_elements()

    /**
     *  This method is what initializes all the elements of the form and adds
     *  them to the FormContent object.
     */
    function form_init_elements()
    {
        $name = new FEText(_("Name"),true);
        $zone = new FEListBox(_("Initial Zone"));
        $zoneinit = array();

        $dbdobj = DB_DataObject::Factory("zone");
        $dbdobj->selectAdd();
        $dbdobj->selectAdd('id');
        $dbdobj->selectAdd('name');
        $dbdobj->orderBy('name ASC');

        $dbdobj->find();

        //start grabbing zones
        while($dbdobj->fetch())
            $zoneinit[$dbdobj->name] = $dbdobj->id;
        
        $zone->set_list_data($zoneinit);
        
        $this->add_elements($name, $zone);
        $this->set_action($_SERVER['SCRIPT_NAME']);        
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method is for validation to assure that the processing method doesn't 
     *  deal with bad data.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true;
        
        $wg = new Workgroup();
        $vals = $this->get_all_element_values();

        $wg->setName($vals[_("Name")]);
        
        $valid = $wg->validateName();
       
        //boolean true means perfectly valid
        if($valid !== true) {
            //errors encountered, remap to form values
            $errors = $this->remap($valid, FORM);
            foreach($errors as $label=>$error) {
                if($error !== true) //make sure that form item was wrong
                    $this->add_error($label,$error); //report error
            }
            $res = false; //not valid
        }

        return $res;
    }

//}}}
//{{{   form_action()

    /**
     *  This method performs the appropriate actions with the form data after
     *  validation has returned true.
     *
     *  @return Boolean $res    True if action completed successfully
     */
    function form_action()
    {
        global $user;

        $res = true;
    
        $zwg = DB_DataObject::Factory('zone_workgroup');
        $vals = $this->get_all_element_values();
        $zwg->setFrom($this->remap($vals, DBDO));
        
        $wg = new Workgroup();
        $wg->name = $vals[_('Name')];
        $wg->query("BEGIN");

        //This looks kind of weird but it insert returns the id, and that's
        //what the workgroup wants.
        $zwg->workgroup = $wg->insert();

        if($zwg->insert() === false) {
            $wh->query("ROLLBACK");
            $this->add_error(_("ERROR"),_("Workgroup failed to be written to DB"));
            $res = false;
        } else {
            $wg->query("COMMIT");
            $wg->write_history(_("Created Workgroup"), ACTION_CREATE, $user->username, $zwg->zone);
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method is what gets run right after the form_action if the action
     *  was performed successfully
     *
     *  @return DIVTag  $div    A success message and links to get back home
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a("index.php",_("[Return to Home]"));
        $wglink = html_a("workgroup.php", _("[Return to Workgroup List]"));
        
        $div->add(container(_("Workgroup successfully created!"),html_br()));
        $div->add(container($indlink, $wglink));

        return $div;
    }

//}}}
//{{{   form_buttons()

    /** 
     *  This method adds the buttons to the form's table
     *
     *  @param  String  $class  The class for the table row
     *
     *  @return TRTag   $tr     The table row with the buttons
     */
    function form_buttons($class)
    {
        $this->set_cancel_action($_SERVER["SCRIPT_NAME"]);
        $tr = html_tr($class);
        $tr->add("&nbsp;",container($this->add_action(_("Create")), $this->add_cancel()));
//$this->add_action(_("Cancel"))));
        return $tr;
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method adds the hidden elements to the form
     *
     *  @return FEHidden        Hidden element for the action
     */
    function form_hidden()
    {
        $hidden = new FEHidden("action","create");
        $this->add_element($hidden);
        return $this->element_form("action");
    }

//}}}

}

?>
