<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: auth_maintain.php 4408 2006-05-11 20:28:52Z claym $

require_once INCLUDEDIR."/auth.php";

/**
  * The default Maintain user authentication module
  *
  * @package    modules
  * @subpackage maintain_auth
  */
class Auth_Maintain extends AuthModule
{

/* Module Overloads */
// {{{ constructor

    function Auth_Maintain()
    {
        $this->_path = dirname(__FILE__);
        $this->config = $conf['database'];
        $this->_container = new MaintAuth_Container_DB();
    }

// }}}

// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {
        return true;
    }

// }}}

/* AuthMod Overloads */
// {{{ auth()

    function auth($username, $password) 
    {
        global $conf;

        // we need the PEAR_Auth extended classes
        require_once CLASSDIR.'/auth.php';

        // instantiate the Auth object with the given container
        $a = new MaintAuth($this->_container, '', '', false); 

        // set the username and password (Pear::Auth demands $_POST vars by default)
        $a->setAuthVars($username, $password);

        // start the session, and attempt to authenticate the user
        $a->start();

        // if the user was authenticated, we're done
        if ($a->checkAuth()) {
            $this->_status = _("success");
            return true;
        }

        // otherwise, let's explain what went wrong
        switch($a->getStatus()) {

        case AUTH_WRONG_LOGIN:
            $this->_status = $conf['auth']['login_failed'];
            break;  

        default:
            $this->_status = $conf['auth']['bad_auth_cred'];
            break;  
        }

        return false;
    }

// }}}
// {{{ hasUser()

    function hasUser($username)
    {
        if ($this->_container->getUser($username)) {
            return true;
        }

        return false;
    }

// }}}

} //Auth_Maintain class

require_once "Auth/Container/DB.php";

/**
  * Overloaded methods from the Database container in PEAR_Auth
  * allowing us to do certain tricks like determining the password
  * hash to use based on MySQL version, etc.
  *
  * @package    classes
  * @subpackage auth
  */
class MaintAuth_Container_DB extends Auth_Container_DB {

//{{{ Constructor

/**
 * Builds a proper Auth_Container_DB object
 * based on stored config values
 */
function MaintAuth_Container_DB() 
{
    global $conf;
    $params = array(
        'usernamecol'   => 'username',
        'passwordcol'   => 'password',
        'dsn'           => 'mysql://'.$conf['database']['username'].':'
                            .$conf['database']['password'].'@'.
                            $conf['database']['host'].'/'.$conf['database']['database'],
        'cryptType'     => 'sha1', // the only hash currently supported: old ones are in the auth_legacy module
        'table'         => 'user');
    parent::Auth_Container_DB($params);
}

//}}}
//{{{ getUser()

    /**
      * Overloading an as-of-yet unwritten function in Auth_Container_DB, so
      * currently users Auth_Container's getUser() function.  This function
      * reads a list of all users on the server, and searches the array
      * for the given username.
      * This is a terrible method of doing it, so a direct search for a given
      * username, using a generated SQL query, is done instead
      *
      * @param  string  $username   Name of user to search for
      * @return boolean true if username was found
      */
    function getUser($username) {
        if(!$username) 
            return false;

        $err = $this->_prepare();
        if ($err !== true) 
            return PEAR::raiseError($err->getMessage(), $err->getCode());

        $query = sprintf("SELECT %s FROM %s WHERE %s='%s' AND type='mysql'",
                        $this->options['usernamecol'],
                        $this->options['table'],
                        $this->options['usernamecol'],
                        $username
                        );
        $res = $this->db->getOne($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($res))
            return PEAR::raiseError($res->getMessage(), $res->getCode());
        else 
            return ($res == NULL?false:true);

    }

//}}}

} // MaintAuth_Container

?>
